# Used in setup.py
# -*- coding: utf-8 -*-
VERSION = "0.0.11"
PROJECT_PACKAGE_NAME = "pypca"
PROJECT_LICENSE = "MIT"
PROJECT_URL = "http://www.github.com/majuss/pypca"
PROJECT_DESCRIPTION = "A python cli for PCA 301 smart plugs."
PROJECT_LONG_DESCRIPTION = (
    "pypca is a python3 interface for"
    " the ELV PCA 301 smart plug."
    " Its intented to get used in various"
    " smart home services to get a full"
    " integration of all you devices."
)
PROJECT_AUTHOR = "Majuss"

SEND_SUFFIX = "s"
DISCOVERY_TIME = 15
DISCOVERY_TIMEOUT = 30